// const _text_styles_ = ["bold", "underline", "italic", "sup", "sub"];

let _isPreview = true;
let _container;

function transformToLocalResourcePath(value) {
  return value;
}

/**
 * 将富文本 JSON 渲染到指定的元素中
 *
 * @param {RichTextJSON} body
 * @param {HTMLDivElement} container
 */
export function renderRichText(body, container, isPreview = true) {
  _isPreview = isPreview;
  _container = container;
  let sections = body?.sections || [];
  let nodes = [];
  sections.forEach((section) => {
    nodes.push(renderSection(section));
  });
  if (container != undefined) {
    // container.classList.add("rich-text");
    while (container.hasChildNodes()) {
      container.removeChild(container.lastChild);
    }
    nodes.forEach((node) => {
      container.appendChild(node);
    });
  }
}

/**
 * @param {RichTextSectionJSON} section
 * @returns {HTMLDivElement} 返回根据 section 渲染好的 HTMLDivElement
 */
function renderSection(section) {
  let blocks = section.blocks || [];
  let inline = blocks.length > 1;
  let node = document.createElement("div");
  // node.style = "display: flex;";
  blocks.forEach((block) => {
    node.appendChild(renderBlock(block, inline));
  });
  return node;
}

/**
 * @param {RichTextBlockJSON} block
 * @param {Boolean} inline 图片是否以 inline 的样式展示
 * @returns {HTMLElement} 返回根据 block 渲染好的 HTMLElement
 */
function renderBlock(block, inline) {
  // let node = document.createElement('span')
  // let classList = node.classList
  let node;
  if (block.type === "text") {
    // classList.add('text')
    // if (block.param != undefined) {
    //     _text_styles_.forEach(style => {
    //         if (block.param[style] === true) {
    //             classList.add(style)
    //         }
    //     })
    // }
    if (block.param) {
      let uNode, bNode, iNode;
      if (block.param.underline) {
        uNode = document.createElement("u");
      }
      if (block.param.bold) {
        bNode = document.createElement("b");
      }
      if (block.param.italic) {
        iNode = document.createElement("i");
      }
      // 将不为空的元素依次append
      node = [uNode, bNode, iNode]
        .filter((v) => v)
        .reduceRight((p, c) => {
          c.appendChild(p);
          return c;
        });

      let childNode = node;

      for (let i = 0; i < 3; i++) {
        if (childNode && childNode.hasChildNodes()) {
          childNode = childNode.childNodes[0];
        }
      }

      childNode.textContent = block.value;
    } else {
      node = document.createTextNode(block.value);
    }
  } else if (block.type === "image") {
    // classList.add("image");
    // classList.add("loading");
    node = document.createElement("img");
    // node.setAttribute("title", "could be a length of audio");
    // node.addEventListener("click", (ev) => {
    //   // console.log(ev);
    //   const shift = ev.shiftKey;
    //   node.setAttribute(
    //     "width",
    //     +getComputedStyle(node).width.replace("px", "") +
    //       (shift ? 1 : -1) +
    //       "px"
    //   );
    // });
    if (!_isPreview) {
      const __container = _container; // because of ES modules static binding
      node.addEventListener("wheel", (ev) => {
        // ev.stopPropagation();
        // console.log(ev);
        const shift = ev.deltaY > 0;
        const newWidth =
          +getComputedStyle(node).width.replace("px", "") + (shift ? 1 : -1);
        // console.log(newWidth, ev.target.naturalWidth);
        if (newWidth >= 16 && newWidth <= ev.target.naturalWidth) {
          node.setAttribute("width", newWidth);
          // console.log(newWidth);
          __container.dispatchEvent(new Event("input"));
        }
      });
    }
    if (inline === true) {
      node.classList.add("inline");
    }

    if (block.value.startsWith("./")) {
      node.src = transformToLocalResourcePath(block.value);
    } else {
      node.src = block.value;
    }

    node.dataset.isImage = true;
    // 公式latex表达式
    if (block.latex) node.dataset.latex = block.latex;
    // param
    if (block.param) {
      node.style.width = block.param.width;
      node.style.height = block.param.height;
    }
  } else if (block.type === "cloze") {
    node = document.createElement("img");

    node.src = `/img/editor/answer_point_${block.value}.png`;
    node.dataset.isAnswerPoint = true;
    node.dataset.order = block.value;
  } else if (block.type === "audio") {
    // classList.add("audio");
    // let audio = node.appendChild(new Audio());
    // audio.src = block.value;
    // audio.controls = true;
    // span 可以被追加文本内容，不好
    // node = document.createElement("span");
    // node.className = "audio";
    // node.dataset["src"] = block.value;
    // node.innerHTML = "&#9654;";
    // // md5 src?
    // node.id = "what";
    // var sheet = window.document.styleSheets[0];
    // sheet.insertRule(
    //   `#what::after{content: "1:03"; font-size: 11px;}`,
    //   sheet.cssRules.length
    // );

    if (_isPreview) {
      node = document.createElement("audio");
      node.className = "audio";
      node.src = transformToLocalResourcePath(block.value);
      node.controls = true;
      node.controlsList = "nodownload";
    } else {
      node = document.createElement("img");
      node.className = "audio";
      node.src = "/img/editor/text_audio.png";
      node.dataset.audioSrc = transformToLocalResourcePath(block.value);
      node.dataset.isAudio = true;
      node.dataset.duration = block.param.duration;
      const mDuration = require("moment").unix(0);
      mDuration.second(block.param.duration);
      node.title = mDuration.format("mm:ss");
    }
  }

  return node;
}

/**
 * @param {Array} content
 * @returns rich json
 */
export function createRichJsonFromText(content) {
  if (content && content.length) {
    const blocks = content.map((cont) => {
      return {
        type: "text",
        value: cont,
      };
    });
    return {
      sections: [
        {
          blocks,
        },
      ],
    };
  } else {
    return {
      sections: [
        {
          blocks: [],
        },
      ],
    };
  }
}

/**
 * 百以下数字转中文汉字
 * @param {Number} num 大于0的100以下数字，其他数值直接转字符串
 *
 * @returns {String}
 */
export function numberToChinese(num) {
  if (num >= 100 || num <= 0) return num + "";
  const cnNums = "一二三四五六七八九十".split("");
  if (num <= 10) return cnNums[num - 1];

  return (num + "")
    .split("")
    .map((item) => item * 1)
    .map((item, index) => {
      if (index) {
        return !item ? "" : cnNums[item - 1];
      } else {
        return item === 1 ? "" : cnNums[item - 1];
      }
    })
    .join("十");
}

/**
 *
 * @param {Number} val 数值，1到26
 */
export function numberToUpperCase(val) {
  if (val < 1 || val > 26) return;

  return String.fromCharCode(64 + val);
}
